
#ifndef AKONADIAGENTWIDGETBASE_EXPORT_H
#define AKONADIAGENTWIDGETBASE_EXPORT_H

#ifdef AKONADIAGENTWIDGETBASE_STATIC_DEFINE
#  define AKONADIAGENTWIDGETBASE_EXPORT
#  define AKONADIAGENTWIDGETBASE_NO_EXPORT
#else
#  ifndef AKONADIAGENTWIDGETBASE_EXPORT
#    ifdef KPim6AkonadiAgentWidgetBase_EXPORTS
        /* We are building this library */
#      define AKONADIAGENTWIDGETBASE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define AKONADIAGENTWIDGETBASE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef AKONADIAGENTWIDGETBASE_NO_EXPORT
#    define AKONADIAGENTWIDGETBASE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef AKONADIAGENTWIDGETBASE_DEPRECATED
#  define AKONADIAGENTWIDGETBASE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef AKONADIAGENTWIDGETBASE_DEPRECATED_EXPORT
#  define AKONADIAGENTWIDGETBASE_DEPRECATED_EXPORT AKONADIAGENTWIDGETBASE_EXPORT AKONADIAGENTWIDGETBASE_DEPRECATED
#endif

#ifndef AKONADIAGENTWIDGETBASE_DEPRECATED_NO_EXPORT
#  define AKONADIAGENTWIDGETBASE_DEPRECATED_NO_EXPORT AKONADIAGENTWIDGETBASE_NO_EXPORT AKONADIAGENTWIDGETBASE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef AKONADIAGENTWIDGETBASE_NO_DEPRECATED
#    define AKONADIAGENTWIDGETBASE_NO_DEPRECATED
#  endif
#endif

#endif /* AKONADIAGENTWIDGETBASE_EXPORT_H */
