/*
    SPDX-FileCopyrightText: 2009 Volker Krause <vkrause@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

/**
  @file capabilities_p.h Shared constants for agent capabilities.

  @todo Fill this file with the missing capabilities.
*/

#define AKONADI_AGENT_CAPABILITY_AUTOSTART "Autostart"
#define AKONADI_AGENT_CAPABILITY_NOCONFIG "NoConfig"
#define AKONADI_AGENT_CAPABILITY_PREPROCESSOR "Preprocessor"
#define AKONADI_AGENT_CAPABILITY_RESOURCE "Resource"
#define AKONADI_AGENT_CAPABILITY_SEARCH "Search"
#define AKONADI_AGENT_CAPABILITY_UNIQUE "Unique"
#define AKONADI_AGENT_CAPABILITY_VIRTUAL "Virtual"
#define AKONADI_AGENT_CAPABILITY_SINGLESHOT "SingleShot"
